@@ -680,10 +680,10 @@ copy_op_data_free (CopyOpData *cod)
 
 
 static void
-catalog_save_done_cb (void     **buffer,
-		      gsize      count,
-		      GError    *error,
-		      gpointer   user_data)
+copy__catalog_save_done_cb (void     **buffer,
+			    gsize      count,
+			    GError    *error,
+			    gpointer   user_data)
 {
 	CopyOpData *cod = user_data;
 
@@ -728,7 +728,7 @@ catalog_ready_cb (GObject  *catalog,
 			    TRUE,
 			    G_PRIORITY_DEFAULT,
 			    NULL,
-			    catalog_save_done_cb,
+			    copy__catalog_save_done_cb,
 			    cod);
 
 	g_object_unref (gio_file);
@@ -1252,6 +1252,143 @@ gth_file_source_catalogs_reorder (GthFileSource *file_source,
 }
 
 
+/* -- gth_catalog_manager_remove_files -- */
+
+
+typedef struct {
+	GtkWindow  *parent;
+	GList      *file_data_list;
+	GFile      *gio_file;
+	GthCatalog *catalog;
+} RemoveFromCatalogData;
+
+
+static void
+remove_from_catalog_end (GError                *error,
+			 RemoveFromCatalogData *data)
+{
+	if (error != NULL)
+		_gtk_error_dialog_from_gerror_show (data->parent, _("Could not remove the files from the catalog"), &error);
+
+	g_object_unref (data->catalog);
+	g_object_unref (data->gio_file);
+	_g_object_list_unref (data->file_data_list);
+	g_free (data);
+}
+
+
+static void
+remove_files__catalog_save_done_cb (void     **buffer,
+				    gsize      count,
+				    GError    *error,
+				    gpointer   user_data)
+{
+	RemoveFromCatalogData *data = user_data;
+
+	if (error == NULL) {
+		GFile *catalog_file;
+		GList *files = NULL;
+		GList *scan;
+
+		catalog_file = gth_catalog_file_from_gio_file (data->gio_file, NULL);
+		for (scan = data->file_data_list; scan; scan = scan->next)
+			files = g_list_prepend (files, g_object_ref (((GthFileData*) scan->data)->file));
+		files = g_list_reverse (files);
+
+		gth_monitor_folder_changed (gth_main_get_default_monitor (),
+					    catalog_file,
+					    files,
+					    GTH_MONITOR_EVENT_REMOVED);
+
+		_g_object_list_unref (files);
+		g_object_unref (catalog_file);
+	}
+
+	remove_from_catalog_end (error, data);
+}
+
+
+static void
+catalog_buffer_ready_cb (void     **buffer,
+			 gsize      count,
+			 GError    *error,
+			 gpointer   user_data)
+{
+	RemoveFromCatalogData *data = user_data;
+	GList                 *scan;
+	void                  *catalog_buffer;
+	gsize                  catalog_size;
+
+	if (error != NULL) {
+		remove_from_catalog_end (error, data);
+		return;
+	}
+
+	data->catalog = gth_hook_invoke_get ("gth-catalog-load-from-data", *buffer);
+	if (data->catalog == NULL) {
+		error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_FAILED, _("Invalid file format"));
+		remove_from_catalog_end (error, data);
+		return;
+	}
+
+	gth_catalog_load_from_data (data->catalog, *buffer, count, &error);
+	if (error != NULL) {
+		remove_from_catalog_end (error, data);
+		return;
+	}
+
+	for (scan = data->file_data_list; scan; scan = scan->next) {
+		GthFileData *file_data = scan->data;
+		gth_catalog_remove_file (data->catalog, file_data->file);
+	}
+
+	catalog_buffer = gth_catalog_to_data (data->catalog, &catalog_size);
+	if (error != NULL) {
+		remove_from_catalog_end (error, data);
+		return;
+	}
+
+	g_write_file_async (data->gio_file,
+			    catalog_buffer,
+			    catalog_size,
+			    TRUE,
+			    G_PRIORITY_DEFAULT,
+			    NULL,
+			    remove_files__catalog_save_done_cb,
+			    data);
+}
+
+
+void
+gth_catalog_manager_remove_files (GtkWindow   *parent,
+				  GthFileData *location,
+				  GList       *file_list)
+{
+	RemoveFromCatalogData *data;
+
+	data = g_new0 (RemoveFromCatalogData, 1);
+	data->parent = parent;
+	data->file_data_list = gth_file_data_list_dup (file_list);
+	data->gio_file = gth_main_get_gio_file (location->file);
+
+	g_load_file_async (data->gio_file,
+			   G_PRIORITY_DEFAULT,
+			   NULL,
+			   catalog_buffer_ready_cb,
+			   data);
+}
+
+
+static void
+gth_file_source_catalogs_remove (GthFileSource *file_source,
+				 GthFileData   *location,
+	       	       	         GList         *file_list /* GthFileData list */,
+	       	       	         gboolean       permanently,
+	       	       	         GtkWindow     *parent)
+{
+	gth_catalog_manager_remove_files (parent, location, file_list);
+}
+
 
 static void
 gth_file_source_catalogs_finalize (GObject *object)
@@ -1293,7 +1430,7 @@ gth_file_source_catalogs_class_init (GthFileSourceCatalogsClass *class)
 	file_source_class->can_cut = gth_file_source_catalogs_can_cut;
 	file_source_class->is_reorderable  = gth_file_source_catalogs_is_reorderable;
 	file_source_class->reorder = gth_file_source_catalogs_reorder;
-
+	file_source_class->remove = gth_file_source_catalogs_remove;
 }
 
 
