@@ -36,12 +36,12 @@ GthMetadataCategory comments_metadata_category[] = {
 
 
 GthMetadataInfo comments_metadata_info[] = {
-	{ "comment::caption", "", "comment", 1, NULL, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::note", "", "comment", 2, NULL, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::place", "", "comment", 3, NULL, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::time", "", "comment", 4, NULL, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::categories", "", "comment", 5, NULL, GTH_METADATA_ALLOW_NOWHERE },
-	{ "comment::rating", "", "comment", 6, NULL, GTH_METADATA_ALLOW_NOWHERE },
+	{ "comment::caption", N_("Title"), "comment", 1, NULL, GTH_METADATA_ALLOW_IN_FILE_LIST },
+	{ "comment::note", N_("Description"), "comment", 2, NULL, GTH_METADATA_ALLOW_IN_FILE_LIST },
+	{ "comment::place", N_("Place"), "comment", 3, NULL, GTH_METADATA_ALLOW_IN_FILE_LIST },
+	{ "comment::time", N_("Date"), "comment", 4, NULL, GTH_METADATA_ALLOW_IN_FILE_LIST },
+	{ "comment::categories", N_("Tags"), "comment", 5, NULL, GTH_METADATA_ALLOW_IN_FILE_LIST },
+	{ "comment::rating", N_("Rating"), "comment", 6, NULL, GTH_METADATA_ALLOW_IN_FILE_LIST },
 	{ NULL, NULL, NULL, 0, 0 }
 };
 
@@ -68,7 +68,7 @@ get_place_for_test (GthTest        *test,
 }
 
 
-void
+static void
 comments__add_sidecars_cb (GFile  *file,
 			   GList **sidecars)
 {
@@ -76,7 +76,7 @@ comments__add_sidecars_cb (GFile  *file,
 }
 
 
-void
+static void
 comments__read_metadata_ready_cb (GthFileData *file_data,
 				  const char  *attributes)
 {
@@ -175,11 +175,16 @@ comments__read_metadata_ready_cb (GthFileData *file_data,
 
 	if (write_comment) {
 		GFile *comment_file;
+		GFile *comment_directory;
 		char  *buffer;
 		gsize  size;
 
-		buffer = gth_comment_to_data (comment, &size);
 		comment_file = gth_comment_get_comment_file (file_data->file);
+		comment_directory = g_file_get_parent (comment_file);
+		if (! g_file_query_exists (comment_directory, NULL))
+			g_file_make_directory (comment_directory, NULL, NULL);
+
+		buffer = gth_comment_to_data (comment, &size);
 		g_write_file (comment_file,
 			      FALSE,
 			      G_FILE_CREATE_NONE,
@@ -188,14 +193,45 @@ comments__read_metadata_ready_cb (GthFileData *file_data,
 			      NULL,
 			      NULL);
 
-		g_object_unref (comment_file);
+		{
+			GFile *parent;
+			GList *list;
+
+			parent = g_file_get_parent (file_data->file);
+			list = g_list_prepend (NULL, file_data->file);
+			gth_monitor_folder_changed (gth_main_get_default_monitor (),
+						    parent,
+						    list,
+						    GTH_MONITOR_EVENT_CHANGED);
+
+			g_list_free (list);
+			g_object_unref (parent);
+		}
+
 		g_free (buffer);
+		g_object_unref (comment_directory);
+		g_object_unref (comment_file);
 	}
 
 	g_object_unref (comment);
 }
 
 
+static void
+comments__delete_metadata_cb (GFile  *file,
+			      void  **buffer,
+			      gsize  *size)
+{
+	GFile *comment_file;
+
+	comment_file = gth_comment_get_comment_file (file);
+	if (comment_file != NULL) {
+		g_file_delete (comment_file, NULL, NULL);
+		g_object_unref (comment_file);
+	}
+}
+
+
 G_MODULE_EXPORT void
 gthumb_extension_activate (void)
 {
@@ -227,6 +263,8 @@ gthumb_extension_activate (void)
 
 	gth_hook_add_callback ("add-sidecars", 10, G_CALLBACK (comments__add_sidecars_cb), NULL);
 	gth_hook_add_callback ("read-metadata-ready", 10, G_CALLBACK (comments__read_metadata_ready_cb), NULL);
+	if (gth_main_extension_is_active ("edit_metadata"))
+		gth_hook_add_callback ("delete-metadata", 10, G_CALLBACK (comments__delete_metadata_cb), NULL);
 }
 
 
