@@ -170,6 +170,16 @@ const char *_KEYWORDS_TAG_NAMES[] = {
 	NULL
 };
 
+/* Some evil camera fill in the ImageDescription or UserComment fields
+   with useless fluff. Try to filter these out, so they do not show up
+   as comments */
+const char *stupid_comment_filter[] = {
+	"OLYMPUS DIGITAL CAMERA",
+	"SONY DSC",
+	"KONICA MINOLTA DIGITAL CAMERA",
+	"MINOLTA DIGITAL CAMERA",
+	NULL };
+
 
 inline static char *
 exiv2_key_from_attribute (const char *attribute)
@@ -458,9 +468,38 @@ set_string_list_attribute_from_tagset (GFileInfo  *info,
 }
 
 
+static void
+clear_studip_comments_from_tagset (GFileInfo  *info,
+				   const char *tagset[])
+{
+	int i;
+
+	for (i = 0; tagset[i] != NULL; i++) {
+		GObject    *metadata;
+		const char *value;
+		int         j;
+
+		metadata = g_file_info_get_attribute_object (info, tagset[i]);
+		if ((metadata == NULL) || ! GTH_IS_METADATA (metadata))
+			continue;
+
+		value = gth_metadata_get_formatted (GTH_METADATA (metadata));
+		for (j = 0; stupid_comment_filter[j] != NULL; j++) {
+			if (strstr (value, stupid_comment_filter[j]) == value) {
+				g_file_info_remove_attribute (info, tagset[i]);
+				break;
+			}
+		}
+	}
+}
+
+
 static void
 set_attributes_from_tagsets (GFileInfo *info)
 {
+	clear_studip_comments_from_tagset (info, _DESCRIPTION_TAG_NAMES);
+	clear_studip_comments_from_tagset (info, _TITLE_TAG_NAMES);
+
 	set_attribute_from_tagset (info, "general::datetime", _LAST_DATE_TAG_NAMES);
 	if (g_file_info_get_attribute_object (info, "general::datetime") == NULL)
 		set_attribute_from_tagset (info, "general::datetime", _ORIGINAL_DATE_TAG_NAMES);
@@ -1104,6 +1143,47 @@ exiv2_write_metadata_to_buffer (void      **buffer,
 }
 
 
+extern "C"
+gboolean
+exiv2_clear_metadata (void   **buffer,
+		      gsize   *buffer_size,
+		      GError **error)
+{
+	try {
+		Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open ((Exiv2::byte*) *buffer, *buffer_size);
+
+		if (image.get() == 0) {
+			if (error != NULL)
+				*error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_FAILED, _("Invalid file format"));
+			return FALSE;
+		}
+
+		try {
+			image->clearMetadata();
+			image->writeMetadata();
+		}
+		catch (Exiv2::AnyError& e) {
+			g_warning ("%s", e.what());
+		}
+
+		Exiv2::BasicIo &io = image->io();
+		io.open();
+		Exiv2::DataBuf buf = io.read(io.size());
+
+		g_free (*buffer);
+		*buffer = g_memdup (buf.pData_, buf.size_);
+		*buffer_size = buf.size_;
+	}
+	catch (Exiv2::AnyError& e) {
+		if (error != NULL)
+			*error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_FAILED, e.what());
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+
 #define MAX_RATIO_ERROR_TOLERANCE 0.01
 
 
@@ -1162,7 +1242,7 @@ exiv2_generate_thumbnail (const char *uri,
 		double ratio_delta = (image_ratio > thumbnail_ratio) ? (image_ratio - thumbnail_ratio) : (thumbnail_ratio - image_ratio);
 
 		if ((ratio_delta > MAX_RATIO_ERROR_TOLERANCE) /* the tolerance is used because the reduced image can have a slightly different ratio due to rounding errors */
-		    || (MAX (pixbuf_width, pixbuf_height) < (requested_size / 2))) /* if the embedded image is too small compared to the requested size */
+		    || (MAX (pixbuf_width, pixbuf_height) < (requested_size * 0.90))) /* ignore the embedded image if it's too small compared to the requested size */
 		{
 			g_object_unref (pixbuf);
 			return NULL;
