@@ -531,7 +531,7 @@ dlg_photo_importer (GthBrowser *browser,
 					"text", SOURCE_LIST_COLUMN_NAME,
 					NULL);
 
-	data->file_list = gth_file_list_new (GTH_FILE_LIST_TYPE_NORMAL, FALSE);
+	data->file_list = gth_file_list_new (gth_icon_view_new (), GTH_FILE_LIST_TYPE_NORMAL, FALSE);
 	sort_type = gth_main_get_sort_type ("file::mtime");
 	gth_file_list_set_sort_func (GTH_FILE_LIST (data->file_list), sort_type->cmp_func, FALSE);
 	gth_file_list_enable_thumbs (GTH_FILE_LIST (data->file_list), TRUE);
@@ -548,7 +548,7 @@ dlg_photo_importer (GthBrowser *browser,
 	general_filter = "file::type::is_media"; /* default value */
 	active_filter = 0;
 
-	data->filter_combobox = gtk_combo_box_new_text ();
+	data->filter_combobox = gtk_combo_box_text_new ();
 	for (i = 0, i_general = -1, scan = tests; scan; scan = scan->next, i++) {
 		const char *registered_test_id = scan->data;
 		GthTest    *test;
@@ -564,7 +564,8 @@ dlg_photo_importer (GthBrowser *browser,
 		}
 
 		data->general_tests = g_list_prepend (data->general_tests, g_strdup (gth_test_get_id (test)));
-		gtk_combo_box_append_text (GTK_COMBO_BOX (data->filter_combobox), gth_test_get_display_name (test));
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (data->filter_combobox),
+						gth_test_get_display_name (test));
 		g_object_unref (test);
 	}
 	data->general_tests = g_list_reverse (data->general_tests);
@@ -625,8 +626,8 @@ dlg_photo_importer (GthBrowser *browser,
 			  "visibility_changed",
 			  G_CALLBACK (file_store_changed_cb),
 			  data);
-	g_signal_connect (G_OBJECT (gth_file_list_get_view (GTH_FILE_LIST (data->file_list))),
-			  "selection_changed",
+	g_signal_connect (gth_file_list_get_view (GTH_FILE_LIST (data->file_list)),
+			  "file-selection-changed",
 			  G_CALLBACK (file_view_selection_changed_cb),
 			  data);
 	data->entry_points_changed_id = g_signal_connect (gth_main_get_default_monitor (),
