@@ -33,7 +33,8 @@
 #include "gth-slideshow.h"
 #include "gth-transition.h"
 
-#define HIDE_CURSOR_DELAY 1000
+#define HIDE_CURSOR_DELAY 1
+#define HIDE_PAUSED_SIGN_DELAY 1
 #define DEFAULT_DELAY 2000
 
 
@@ -57,6 +58,7 @@ struct _GthSlideshowPrivate {
 	GthSlideshowDirection  direction;
 #if HAVE_CLUTTER
 	ClutterTimeline       *timeline;
+	ClutterAlpha          *alpha;
 	ClutterActor          *image1;
 	ClutterActor          *image2;
 	ClutterActor          *paused_actor;
@@ -77,8 +79,11 @@ struct _GthSlideshowPrivate {
 #endif
 	GdkPixbuf             *pause_pixbuf;
 	gboolean               paused;
+	gboolean               paint_paused;
+	guint                  hide_paused_sign;
 	gboolean               animating;
 	gboolean               random_order;
+	GthScreensaver        *screensaver;
 };
 
 
@@ -218,6 +223,11 @@ next_image_cb (gpointer user_data)
 static void
 view_next_image_automatically (GthSlideshow *self)
 {
+	if (self->priv->automatic && ! self->priv->paused)
+		gth_screensaver_inhibit (self->priv->screensaver, GTK_WIDGET (self), _("Playing slideshow"));
+	else
+		gth_screensaver_uninhibit (self->priv->screensaver);
+
 	if (self->priv->automatic) {
 		if (self->priv->next_event != 0)
 			g_source_remove (self->priv->next_event);
@@ -229,14 +239,13 @@ view_next_image_automatically (GthSlideshow *self)
 static void
 image_preloader_requested_ready_cb (GthImagePreloader  *preloader,
 				    GthFileData        *requested,
-				    GdkPixbufAnimation *animation,
+				    GthImage           *image,
 				    int                 original_width,
 				    int                 original_height,
 				    GError             *error,
 				    gpointer            user_data)
 {
 	GthSlideshow *self = user_data;
-	GdkPixbuf    *static_image;
 
 	if (error != NULL) {
 		g_clear_error (&error);
@@ -245,12 +254,7 @@ image_preloader_requested_ready_cb (GthImagePreloader  *preloader,
 	}
 
 	_g_object_unref (self->priv->current_pixbuf);
-
-	static_image = gdk_pixbuf_animation_get_static_image (animation);
-	if (static_image != NULL)
-		self->priv->current_pixbuf = gdk_pixbuf_copy (static_image);
-	else
-		self->priv->current_pixbuf = NULL;
+	self->priv->current_pixbuf = gth_image_get_pixbuf (image);
 
 	if (self->priv->current_pixbuf == NULL) {
 		_gth_slideshow_load_next_image (self);
@@ -281,6 +285,7 @@ gth_slideshow_init (GthSlideshow *self)
 	self->priv->direction = GTH_SLIDESHOW_DIRECTION_FORWARD;
 	self->priv->random_order = FALSE;
 	self->priv->current_pixbuf = NULL;
+	self->priv->screensaver = gth_screensaver_new (NULL);
 
 	self->priv->preloader = gth_image_preloader_new (GTH_LOAD_POLICY_ONE_STEP, 3);
 	g_signal_connect (self->priv->preloader,
@@ -318,6 +323,8 @@ gth_slideshow_finalize (GObject *object)
 	}
 #endif
 
+	_g_object_unref (self->priv->screensaver);
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -470,6 +477,13 @@ _gth_slideshow_construct (GthSlideshow *self,
 							     100,
 							     0,
 							     NULL);
+	if (self->priv->pause_pixbuf == NULL)
+		self->priv->pause_pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+								     GTK_STOCK_MEDIA_PAUSE,
+								     100,
+								     0,
+								     NULL);
+
 	self->priv->projector->construct (self);
 
 	g_signal_connect (self, "show", G_CALLBACK (gth_slideshow_show_cb), self);
@@ -572,7 +586,10 @@ default_projector_image_ready (GthSlideshow *self,
 static void
 default_projector_finalize (GthSlideshow *self)
 {
-	/* void */
+	if (self->priv->hide_paused_sign != 0) {
+		g_source_remove (self->priv->hide_paused_sign);
+		self->priv->hide_paused_sign = 0;
+	}
 }
 
 
@@ -586,6 +603,11 @@ default_projector_hide_cursor (GthSlideshow *self)
 static void
 default_projector_paused (GthSlideshow *self)
 {
+	if (self->priv->hide_paused_sign != 0) {
+		g_source_remove (self->priv->hide_paused_sign);
+		self->priv->hide_paused_sign = 0;
+	}
+	self->priv->paint_paused = TRUE;
 	gtk_widget_queue_draw (self->priv->viewer);
 }
 
@@ -599,7 +621,7 @@ viewer_event_cb (GtkWidget    *widget,
 		gth_image_viewer_show_cursor (GTH_IMAGE_VIEWER (self->priv->viewer));
 		if (self->priv->hide_cursor_event != 0)
 			g_source_remove (self->priv->hide_cursor_event);
-		self->priv->hide_cursor_event = g_timeout_add (HIDE_CURSOR_DELAY, hide_cursor_cb, self);
+		self->priv->hide_cursor_event = g_timeout_add_seconds (HIDE_CURSOR_DELAY, hide_cursor_cb, self);
 	}
 	else if (event->type == GDK_BUTTON_PRESS) {
 		switch (((GdkEventButton *) event)->button) {
@@ -651,6 +673,21 @@ viewer_event_cb (GtkWidget    *widget,
 }
 
 
+static gboolean
+hide_paused_sign_cb (gpointer user_data)
+{
+	GthSlideshow *self = user_data;
+
+	g_source_remove (self->priv->hide_paused_sign);
+	self->priv->hide_paused_sign = 0;
+
+	self->priv->paint_paused = FALSE;
+	gtk_widget_queue_draw (self->priv->viewer);
+
+	return FALSE;
+}
+
+
 static void
 default_projector_pause_painter (GthImageViewer *image_viewer,
 				 GdkEventExpose *event,
@@ -662,24 +699,31 @@ default_projector_pause_painter (GthImageViewer *image_viewer,
 	double        dest_x;
 	double        dest_y;
 
-	if (! self->priv->paused || (self->priv->pause_pixbuf == NULL))
+	if (! self->priv->paused || ! self->priv->paint_paused || (self->priv->pause_pixbuf == NULL))
 		return;
 
 	screen = gtk_widget_get_screen (GTK_WIDGET (image_viewer));
 	if (screen == NULL)
 		return;
 
-	dest_x = gdk_screen_get_width (screen) - gdk_pixbuf_get_width (self->priv->pause_pixbuf) - 10.0;
-	dest_y = gdk_screen_get_height (screen) - gdk_pixbuf_get_height (self->priv->pause_pixbuf) - 10.0;
+	dest_x = (gdk_screen_get_width (screen) - gdk_pixbuf_get_width (self->priv->pause_pixbuf)) / 2.0;
+	dest_y = (gdk_screen_get_height (screen) - gdk_pixbuf_get_height (self->priv->pause_pixbuf)) / 2.0;
 	gdk_cairo_set_source_pixbuf (cr, self->priv->pause_pixbuf, dest_x, dest_y);
 	cairo_rectangle (cr, dest_x, dest_y, gdk_pixbuf_get_width (self->priv->pause_pixbuf), gdk_pixbuf_get_height (self->priv->pause_pixbuf));
 	cairo_fill (cr);
+
+	if (self->priv->hide_paused_sign != 0)
+		g_source_remove (self->priv->hide_paused_sign);
+	self->priv->hide_paused_sign = g_timeout_add_seconds (HIDE_PAUSED_SIGN_DELAY, hide_paused_sign_cb, self);
 }
 
 
 static void
 default_projector_construct (GthSlideshow *self)
 {
+	self->priv->hide_paused_sign = 0;
+	self->priv->paint_paused = FALSE;
+
 	self->priv->viewer = gth_image_viewer_new ();
 	gth_image_viewer_set_black_background (GTH_IMAGE_VIEWER (self->priv->viewer), TRUE);
 	gth_image_viewer_hide_frame (GTH_IMAGE_VIEWER (self->priv->viewer));
@@ -841,7 +885,10 @@ animation_frame_cb (ClutterTimeline *timeline,
 		    GthSlideshow    *self)
 {
 	if (self->priv->transition != NULL)
-		gth_transition_frame (self->priv->transition, self, msecs);
+		gth_transition_frame (self->priv->transition,
+				      self,
+				      clutter_alpha_get_alpha (self->priv->alpha));
+
 	if (self->first_frame)
 		self->first_frame = FALSE;
 }
@@ -938,6 +985,7 @@ static void
 clutter_projector_finalize (GthSlideshow *self)
 {
 	_g_object_unref (self->priv->timeline);
+	_g_object_unref (self->priv->alpha);
 }
 
 
@@ -1138,6 +1186,8 @@ clutter_projector_construct (GthSlideshow *self)
 	g_signal_connect (self->priv->timeline, "new-frame", G_CALLBACK (animation_frame_cb), self);
 	g_signal_connect (self->priv->timeline, "started", G_CALLBACK (animation_started_cb), self);
 
+	self->priv->alpha = clutter_alpha_new_full (self->priv->timeline, CLUTTER_EASE_IN_OUT_SINE);
+
 	if (self->priv->pause_pixbuf != NULL)
 		self->priv->paused_actor = gtk_clutter_texture_new_from_pixbuf (self->priv->pause_pixbuf);
 	else
