@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -341,7 +340,7 @@ gth_filterbar_construct (GthFilterbar *filterbar,
 	GtkWidget       *button;
 	GtkWidget       *image;
 
-	GTK_BOX (filterbar)->spacing = 6;
+	gtk_box_set_spacing (GTK_BOX (filterbar), 6);
 	gtk_container_set_border_width (GTK_CONTAINER (filterbar), 2);
 
 	/* filter combo box */
