@@ -22,7 +22,6 @@
 #include <config.h>
 #include <math.h>
 #include "glib-utils.h"
-#include "gth-cursors.h"
 #include "gth-image-dragger.h"
 
 
@@ -67,17 +66,20 @@ gth_image_dragger_instance_init (GthImageDragger *dragger)
 
 
 static void
-_gth_image_dragger_update_cursor (GthImageDragger *self)
+gth_image_dragger_set_viewer (GthImageViewerTool *base,
+			      GthImageViewer     *image_viewer)
 {
-	GdkCursor *cursor;
+	GthImageDragger *self = GTH_IMAGE_DRAGGER (base);
+	self->priv->viewer = image_viewer;
+}
 
-	if (self->priv->draggable)
-		cursor = gth_cursor_get (gtk_widget_get_window (GTK_WIDGET (self->priv->viewer)), GTH_CURSOR_HAND_OPEN);
-	else
-		cursor = gdk_cursor_new (GDK_LEFT_PTR);
-	gth_image_viewer_set_cursor (self->priv->viewer, cursor);
 
-	gdk_cursor_unref (cursor);
+static void
+gth_image_dragger_unset_viewer (GthImageViewerTool *base,
+		       	         GthImageViewer     *image_viewer)
+{
+	GthImageDragger *self = GTH_IMAGE_DRAGGER (base);
+	self->priv->viewer = NULL;
 }
 
 
@@ -95,6 +97,18 @@ gth_image_dragger_unrealize (GthImageViewerTool *base)
 }
 
 
+static void
+_gth_image_dragger_update_cursor (GthImageDragger *self)
+{
+	GdkCursor *cursor;
+
+	cursor = gdk_cursor_new (GDK_LEFT_PTR);
+	gth_image_viewer_set_cursor (self->priv->viewer, cursor);
+
+	gdk_cursor_unref (cursor);
+}
+
+
 static void
 gth_image_dragger_size_allocate (GthImageViewerTool *base,
 				 GtkAllocation      *allocation)
@@ -141,30 +155,34 @@ gth_image_dragger_expose (GthImageViewerTool *self,
 {
 	GthImageDragger *dragger;
 	GthImageViewer  *viewer;
-	GdkInterpType    interp_type;
+	cairo_filter_t   filter;
 
 	dragger = (GthImageDragger *) self;
 	viewer = dragger->priv->viewer;
 
-	if (gth_image_viewer_get_current_pixbuf (viewer) == NULL)
+	gth_image_viewer_paint_background (viewer, cr);
+
+	if (gth_image_viewer_get_current_image (viewer) == NULL)
 		return;
 
 	if (gth_image_viewer_get_zoom_quality (viewer) == GTH_ZOOM_QUALITY_LOW)
-		interp_type = GDK_INTERP_TILES;
+		filter = CAIRO_FILTER_FAST;
 	else
-		interp_type = GDK_INTERP_BILINEAR;
+		filter = CAIRO_FILTER_BEST;
 
 	if (gth_image_viewer_get_zoom (viewer) == 1.0)
-		interp_type = GDK_INTERP_NEAREST;
+		filter = CAIRO_FILTER_FAST;
 
 	gth_image_viewer_paint_region (viewer,
 				       cr,
-				       gth_image_viewer_get_current_pixbuf (viewer),
+				       gth_image_viewer_get_current_image (viewer),
 				       viewer->x_offset - viewer->image_area.x,
 				       viewer->y_offset - viewer->image_area.y,
 				       &viewer->image_area,
 				       event->region,
-				       interp_type);
+				       filter);
+
+	gth_image_viewer_apply_painters (viewer, event, cr);
 }
 
 
@@ -187,7 +205,7 @@ gth_image_dragger_button_press (GthImageViewerTool *self,
 		GdkCursor *cursor;
 		int        retval;
 
-		cursor = gth_cursor_get (gtk_widget_get_window (widget), GTH_CURSOR_HAND_CLOSED);
+		cursor = gdk_cursor_new_from_name (gtk_widget_get_display (widget), "grabbing");
 		retval = gdk_pointer_grab (gtk_widget_get_window (widget),
 					   FALSE,
 					   (GDK_POINTER_MOTION_MASK
@@ -196,7 +214,9 @@ gth_image_dragger_button_press (GthImageViewerTool *self,
 					   NULL,
 					   cursor,
 					   event->time);
-		gdk_cursor_unref (cursor);
+
+		if (cursor != NULL)
+			gdk_cursor_unref (cursor);
 
 		if (retval != 0)
 			return FALSE;
@@ -274,6 +294,8 @@ gth_image_dragger_zoom_changed (GthImageViewerTool *self)
 static void
 gth_image_dragger_gth_image_tool_interface_init (GthImageViewerToolIface *iface)
 {
+	iface->set_viewer = gth_image_dragger_set_viewer;
+	iface->unset_viewer = gth_image_dragger_unset_viewer;
 	iface->realize = gth_image_dragger_realize;
 	iface->unrealize = gth_image_dragger_unrealize;
 	iface->size_allocate = gth_image_dragger_size_allocate;
@@ -323,12 +345,7 @@ gth_image_dragger_get_type (void)
 
 
 GthImageViewerTool *
-gth_image_dragger_new (GthImageViewer *viewer)
+gth_image_dragger_new (void)
 {
-	GthImageDragger *dragger;
-
-	dragger = g_object_new (GTH_TYPE_IMAGE_DRAGGER, NULL);
-	dragger->priv->viewer = viewer;
-
-	return GTH_IMAGE_VIEWER_TOOL (dragger);
+	return (GthImageViewerTool *) g_object_new (GTH_TYPE_IMAGE_DRAGGER, NULL);
 }
