@@ -263,8 +263,17 @@ pixbuf_task_save_current_pixbuf (GthPixbufListTask *self,
 	if (file != NULL)
 		gth_file_data_set_file (self->priv->destination_file_data, file);
 
+	if (GTH_PIXBUF_TASK (self->priv->task)->dest == NULL) {
+		process_next_file (self);
+		return;
+	}
+
+	/* add a reference before unref-ing new_pixbuf because dest and
+	 * new_pixbuf can be the same object. */
+
+	g_object_ref (GTH_PIXBUF_TASK (self->priv->task)->dest);
 	_g_object_unref (self->priv->new_pixbuf);
-	self->priv->new_pixbuf = g_object_ref (GTH_PIXBUF_TASK (self->priv->task)->dest);
+	self->priv->new_pixbuf = GTH_PIXBUF_TASK (self->priv->task)->dest;
 
 	_gdk_pixbuf_save_async (self->priv->new_pixbuf,
 				self->priv->destination_file_data,
@@ -407,6 +416,8 @@ process_current_file (GthPixbufListTask *self)
 	GList       *source_singleton;
 
 	if (self->priv->current == NULL) {
+		if (self->priv->destination_folder != NULL)
+			gth_browser_go_to (self->priv->browser, self->priv->destination_folder, NULL);
 		gth_task_completed (GTH_TASK (self), NULL);
 		return;
 	}
