@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef GTK_UTILS_H
@@ -77,6 +76,10 @@ _gtk_error_dialog_from_gerror_show         (GtkWindow        *parent,
 void        _gtk_error_dialog_run          (GtkWindow        *parent,
 					    const gchar      *format,
 					    ...) G_GNUC_PRINTF (2, 3);
+void        _gtk_error_dialog_show         (GtkWindow        *parent,
+					    const char       *title,
+					    const char       *format,
+					    ...) G_GNUC_PRINTF (3, 4);
 void        _gtk_info_dialog_run           (GtkWindow        *parent,
 					    const gchar      *format,
 					    ...) G_GNUC_PRINTF (2, 3);
@@ -105,7 +108,7 @@ GtkWidget *
 			 		    const char       *name);
 GtkWidget * _gtk_combo_box_new_with_texts  (const char       *first_text,
 					    ...);
-void        _gtk_combo_box_append_texts    (GtkComboBox      *combo_box,
+void        _gtk_combo_box_append_texts    (GtkComboBoxText  *combo_box,
 					    const char       *first_text,
 					    ...);
 GtkWidget * _gtk_image_new_from_xpm_data   (char             *xpm_data[]);
@@ -126,6 +129,18 @@ void        _gtk_window_resize_to_fit_screen_height
 					    int               default_width);
 void        _gtk_info_bar_clear_action_area (GtkInfoBar      *info_bar);
 
+GdkDragAction _gtk_menu_ask_drag_drop_action (GtkWidget     *widget,
+					      GdkDragAction  actions,
+					      guint32        activate_time);
+GtkAdjustment * gimp_scale_entry_new         (GtkWidget *parent_box,
+		      	      	      	      GtkLabel  *label,
+		      	      	      	      float      value,
+		      	      	      	      float      lower,
+		      	      	      	      float      upper,
+		      	      	      	      float      step_increment,
+		      	      	      	      float      page_increment,
+		      	      	      	      int        digits);
+
 G_END_DECLS
 
 #endif
